@@ -145,7 +145,7 @@ class Morphology
        If recursive == true, all descendent sections will be deleted as well
        Else, children will be re-attached to their grand-parent
     **/
-    void deleteSection(const std::shared_ptr<Section>& section, bool recursive = true);
+    void deleteSection(std::shared_ptr<Section> section, bool recursive = true);
 
     /**
        Append the existing morphio::Section as a root section
@@ -210,8 +210,13 @@ class Morphology
        Fixes the morphology single child sections and issues warnings
        if the section starts and ends are inconsistent
      **/
-    void sanitize();
-    void sanitize(const morphio::readers::DebugInfo& debugInfo);
+    void removeUnifurcations();
+    void removeUnifurcations(const morphio::readers::DebugInfo& debugInfo);
+
+    /**
+       Used before writing to SWC to check that there is no unifurcation
+     **/
+    void _raiseIfUnifurcations();
 
   public:
     friend class Section;
@@ -233,6 +238,10 @@ class Morphology
 
     std::map<uint32_t, uint32_t> _parent;
     std::map<uint32_t, std::vector<std::shared_ptr<Section>>> _children;
+
+  private:
+    void eraseByValue(std::vector<std::shared_ptr<Section>>& vec,
+                      const std::shared_ptr<Section> section);
 };
 
 inline const std::vector<std::shared_ptr<Section>>& Morphology::rootSections() const noexcept {
