@@ -1,24 +1,22 @@
-import os
 from collections import OrderedDict
 
 import numpy as np
-from numpy.testing import assert_equal
-from nose.tools import assert_dict_equal, assert_raises, ok_
-from numpy.testing import assert_array_equal
 from pathlib import Path
 from tempfile import TemporaryDirectory
 
 import morphio
-from morphio import MitochondriaPointLevel, MorphioError, RawDataError
+from morphio import CellFamily, IterType, MitochondriaPointLevel, MorphioError
 from morphio import Morphology as ImmutableMorphology
-from morphio import (PointLevel, SectionBuilderError, SectionType,
-                     IterType, ostream_redirect, CellFamily)
-from morphio.mut import Morphology, GlialCell
-from . utils import assert_substring, captured_output, tmp_asc_file, setup_tempdir
+from morphio import PointLevel, RawDataError, SectionBuilderError, SectionType, ostream_redirect
+from morphio.mut import GlialCell, Morphology
+from nose.tools import assert_dict_equal, assert_raises, ok_
+from numpy.testing import assert_array_equal, assert_equal
 
-_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
+from .utils import assert_substring, captured_output, setup_tempdir, tmp_asc_file
 
-SIMPLE = Morphology(os.path.join(_path, "simple.swc"))
+DATA_DIR = Path(__file__).parent / "data"
+
+SIMPLE = Morphology(Path(DATA_DIR, "simple.swc"))
 
 
 def test_point_level():
@@ -47,9 +45,9 @@ def test_point_level():
 
 def test_connectivity():
     cells = OrderedDict({
-        'asc': Morphology(os.path.join(_path, "simple.asc")),
-        'swc': Morphology(os.path.join(_path, "simple.swc")),
-        'h5': Morphology(os.path.join(_path, "h5/v1/simple.h5")),
+        'asc': Morphology(Path(DATA_DIR, "simple.asc")),
+        'swc': Morphology(Path(DATA_DIR, "simple.swc")),
+        'h5': Morphology(Path(DATA_DIR, "h5/v1/simple.h5")),
     })
 
     for cell in cells:
@@ -157,7 +155,7 @@ def test_append_no_duplicate():
 
 
 def test_mut_copy_ctor():
-    simple = Morphology(os.path.join(_path, "simple.swc"))
+    simple = Morphology(Path(DATA_DIR, "simple.swc"))
     assert_equal([sec.id for sec in simple.iter()],
                  [0, 1, 2, 3, 4, 5])
     copy = Morphology(simple)
@@ -230,13 +228,13 @@ def test_build_read_only():
 
 
 def test_mutable_immutable_equivalence():
-    morpho = ImmutableMorphology(os.path.join(_path, "simple.swc"))
+    morpho = ImmutableMorphology(Path(DATA_DIR, "simple.swc"))
     assert_array_equal(morpho.points, morpho.as_mutable().as_immutable().points)
 
 
 def test_mitochondria_read():
-    '''Read a H5 file with a mitochondria'''
-    morpho = Morphology(os.path.join(_path, "h5/v1/mitochondria.h5"))
+    """Read a H5 file with a mitochondria"""
+    morpho = Morphology(DATA_DIR / "h5/v1/mitochondria.h5")
     mito = morpho.mitochondria
     assert_equal(len(mito.root_sections), 2)
 
@@ -265,10 +263,10 @@ def test_mitochondria_read():
 
 
 def test_sections_are_not_dereferenced():
-    '''There used to be a bug where if you would call:
+    """There used to be a bug where if you would call:
     mitochondria.sections, that would dereference all section pointers
-    if mitochondria.sections was not kept in a variable'''
-    morpho = Morphology(os.path.join(_path, "h5/v1/mitochondria.h5"))
+    if mitochondria.sections was not kept in a variable"""
+    morpho = Morphology(Path(DATA_DIR, "h5/v1/mitochondria.h5"))
 
     # This lines used to cause a bug
     morpho.mitochondria.sections  # pylint: disable=pointless-statement
@@ -344,7 +342,7 @@ def test_iterators():
     assert_array_equal([sec.id for sec in SIMPLE.iter(IterType.breadth_first)],
                        [0, 3, 1, 2, 4, 5])
 
-    neuron = Morphology(os.path.join(_path, "iterators.asc"))
+    neuron = Morphology(Path(DATA_DIR, "iterators.asc"))
     root = neuron.root_sections[0]
     assert_array_equal([section.id for section in root.iter(IterType.depth_first)],
                        [0, 1, 2, 3, 4, 5, 6])
@@ -355,7 +353,7 @@ def test_iterators():
                        [0, 7, 1, 4, 8, 9, 2, 3, 5, 6])
 
 def test_non_C_nparray():
-    m = Morphology(os.path.join(_path, "simple.swc"))
+    m = Morphology(Path(DATA_DIR, "simple.swc"))
     section = m.root_sections[0]
     points = np.array([[1, 2, 3], [4, 5, 6]])
     section.points = points
@@ -374,7 +372,7 @@ def test_non_C_nparray():
 def test_annotation():
     with captured_output() as (_, err):
         with ostream_redirect(stdout=True, stderr=True):
-            with tmp_asc_file('''((Dendrite)
+            with tmp_asc_file("""((Dendrite)
                       (3 -4 0 2)
                       (3 -6 0 2)
                       (3 -8 0 2)
@@ -386,26 +384,86 @@ def test_annotation():
                         |       ; <-- empty sibling but still works !
                        )
                       )
-                 ''') as tmp_file:
+                 """) as tmp_file:
                 cell = Morphology(tmp_file.name)
+                cell.remove_unifurcations()
 
     for n in (cell, cell.as_immutable(), cell.as_immutable().as_mutable()):
         assert_equal(len(n.annotations), 1)
         annotation = n.annotations[0]
         assert_equal(annotation.type, morphio.AnnotationType.single_child)
 
+def test_empty_sibling():
+    '''The empty sibling will be removed and the single child will be merged
+    with its parent'''
+    with captured_output() as (_, err):
+        with ostream_redirect(stdout=True, stderr=True):
+            with tmp_asc_file('''((Dendrite)
+                      (3 -4 0 10)
+                      (3 -6 0 9)
+                      (3 -8 0 8)
+                      (3 -10 0 7)
+                      (
+                        (3 -10 0 6)
+                        (0 -10 0 5)
+                        (-3 -10 0 4)
+                        |       ; <-- empty sibling but still works !
+                       )
+                      )
+                 ''') as tmp_file:
+                n = Morphology(tmp_file.name)
+                n.remove_unifurcations()
+                assert_substring('is the only child of section: 0',
+                                 err.getvalue().strip())
+                assert_substring('It will be merged with the parent section',
+                                 err.getvalue().strip())
+
+    assert_equal(len(n.root_sections), 1)
+    assert_array_equal(n.root_sections[0].points,
+                       np.array([[3, -4, 0],
+                                 [3, -6, 0],
+                                 [3, -8, 0],
+                                 [3, -10, 0],
+                                 [0, -10, 0],
+                                 [-3, -10, 0]],
+                                dtype=np.float32))
+    assert_array_equal(n.root_sections[0].diameters,
+                       np.array([10, 9, 8, 7, 5, 4], dtype=np.float32))
+
+    assert_equal(len(n.annotations), 1)
+    annotation = n.annotations[0]
+    assert_equal(annotation.type, morphio.AnnotationType.single_child)
+    assert_equal(annotation.line_number, -1)
+    assert_array_equal(annotation.points, [[3, -10, 0], [0, -10, 0], [-3, -10, 0]])
+    assert_array_equal(annotation.diameters, [6, 5, 4])
+
+
+def test_nested_single_child():
+    with captured_output() as (_, err):
+        with ostream_redirect(stdout=True, stderr=True):
+            n = Morphology(DATA_DIR / 'nested_single_children.asc')
+            n.remove_unifurcations()
+    assert_array_equal(n.root_sections[0].points,
+                       [[0., 0., 0.],
+                        [0., 0., 1.],
+                        [0., 0., 2.],
+                        [0., 0., 3.],
+                        [0., 0., 4.]])
+    assert_array_equal(n.root_sections[0].diameters, np.array([8, 7, 6, 5, 4], dtype=np.float32))
+
+
 def test_section___str__():
     assert_equal(str(SIMPLE.root_sections[0]),
                  'Section(id=0, points=[(0 0 0),..., (0 5 0)])')
 
 
 def test_from_pathlib():
-    neuron = Morphology(Path(_path, "simple.asc"))
+    neuron = Morphology(DATA_DIR / "simple.asc")
     assert_equal(len(neuron.root_sections), 2)
 
 
 def test_endoplasmic_reticulum():
-    neuron = Morphology(Path(_path, "simple.asc"))
+    neuron = Morphology(DATA_DIR / "simple.asc")
     reticulum = neuron.endoplasmic_reticulum
     assert_equal(reticulum.section_indices, [])
     assert_equal(reticulum.volumes, [])
@@ -434,7 +492,7 @@ def test_endoplasmic_reticulum():
     assert_equal(reticulum.filament_counts, [4, 4])
 
 
-def test_sanitize():
+def test_remove_unifurcations():
     m = Morphology()
     section = m.append_root_section(PointLevel([[1, 0, 0],
                                                 [2, 0, 0]], [2, 2], [20, 20]),
@@ -443,13 +501,13 @@ def test_sanitize():
                                        [3, 0, 0]], [2, 2], [20, 20]))
     with captured_output() as (_, err):
         with ostream_redirect(stdout=True, stderr=True):
-            m.sanitize()
+            m.remove_unifurcations()
             assert_equal(len(list(m.iter())), 1)
             assert_equal(err.getvalue().strip(),
                          'Warning: section 1 is the only child of section: 0\nIt will be merged '
                          'with the parent section')
 
-    # Checking that sanitize() issues a warning on missing duplicate
+    # Checking that remove_unifurcations() issues a warning on missing duplicate
     m = Morphology()
     section = m.append_root_section(PointLevel([[1, 0, 0],
                                                 [2, 0, 0]], [2, 2], [20, 20]),
@@ -462,29 +520,106 @@ def test_sanitize():
                                                [2, 0, 0]], [2, 2], [20, 20]))
     with captured_output() as (_, err):
         with ostream_redirect():
-            m.sanitize()
+            m.remove_unifurcations()
             assert_equal(err.getvalue().strip(),
                          'Warning: while appending section: 2 to parent: 0\nThe section first point should be parent section last point: \n        : X Y Z Diameter\nparent last point :[2.000000, 0.000000, 0.000000, 2.000000]\nchild first point :[2.000000, 1.000000, 0.000000, 2.000000]')
 
 
+def test_remove_rootsection():
+    morpho = Morphology(DATA_DIR / 'single_point_root.asc')
+    assert_equal(len(morpho.root_sections), 1)
+    to_remove = []
+    for root in morpho.root_sections:
+        if len(root.points) == 1:
+            to_remove.append(root)
+    for root in to_remove:
+        morpho.delete_section(root, False)
+    assert_equal(len(morpho.root_sections), 2)
+
+
+def test_remove_rootsection_in_loop():
+    morpho = Morphology(DATA_DIR / 'single_point_root.asc')
+    assert_equal(len(morpho.root_sections), 1)
+    for root in morpho.root_sections:
+        if len(root.points) == 1:
+            morpho.delete_section(root, False)
+    assert_equal(len(morpho.root_sections), 2)
+
+
 def test_glia():
     g = GlialCell()
     assert_equal(g.cell_family, CellFamily.GLIA)
 
-    g = GlialCell(os.path.join(_path, 'astrocyte.h5'))
+    g = GlialCell(Path(DATA_DIR, 'astrocyte.h5'))
     assert_equal(g.cell_family, CellFamily.GLIA)
 
-    g = GlialCell(Path(_path, 'astrocyte.h5'))
+    g = GlialCell(DATA_DIR / 'astrocyte.h5')
     assert_equal(g.cell_family, CellFamily.GLIA)
 
-    assert_raises(RawDataError, GlialCell, Path(_path, 'simple.swc'))
-    assert_raises(RawDataError, GlialCell, Path(_path, 'h5/v1/simple.h5'))
+    assert_raises(RawDataError, GlialCell, DATA_DIR / 'simple.swc')
+    assert_raises(RawDataError, GlialCell, DATA_DIR / 'h5/v1/simple.h5')
 
 
 def test_glia_round_trip():
     with TemporaryDirectory() as folder:
-        g = GlialCell(os.path.join(_path, 'astrocyte.h5'))
+        g = GlialCell(Path(DATA_DIR, 'astrocyte.h5'))
         filename = Path(folder, 'glial-cell.h5')
         g.write(filename)
         g2 = GlialCell(filename)
         assert_equal(len(g.sections), len(g2.sections))
+
+
+def _get_section():
+    """This is used so that the reference to m is destroyed."""
+    m = Morphology(DATA_DIR / 'simple.swc')
+    s = m.root_sections[0]
+    return s
+
+
+def test_lifetime_destroyed_morphology():
+    """Accessing topological info after a Morphology has been destroyed should raise."""
+    m = Morphology(DATA_DIR / 'simple.swc')
+    s = m.root_sections[0]
+
+    del m  # ~mut.Morphology() called
+
+    with assert_raises(RuntimeError):
+        s.children
+
+
+def test_lifetime_access_properties_no_morphology():
+    """Accessing topological info after a Morphology has been destroyed should raise but
+    "self properties" should be available.
+    """
+    section = _get_section()
+    assert_array_equal(section.points,
+                       np.array([[0., 0., 0.],
+                                 [0., 5., 0.]], dtype=np.float32))
+
+    with assert_raises(RuntimeError):
+        section.children
+
+
+def test_lifetime_copy_single_section():
+    """Copying a single section from a destroyed morphology works because it
+    does not use any topological information"""
+    section = _get_section()
+
+    # Proof that the morphology has really been destroyed
+    with assert_raises(RuntimeError):
+        section.children
+
+    morph = Morphology()
+    morph.append_root_section(section)
+    del section
+    assert_equal(len(morph.root_sections), 1)
+    assert_array_equal(morph.root_sections[0].points,
+                       np.array([[0., 0., 0.],
+                                 [0., 5., 0.]], dtype=np.float32))
+
+
+def test_lifetime_iteration_fails_with_orphan_section():
+    section = _get_section()
+    for iter_type in IterType.depth_first, IterType.breadth_first, IterType.upstream:
+        with assert_raises(RuntimeError):
+            section.iter(iter_type)
